@@ -1,7 +1,6 @@
 #!/bin/bash
 
 # OPTIONS
-claspre=0
 static=0
 machine=0
 mt=0
@@ -37,9 +36,6 @@ while [[ $# > 0 ]]; do
 		"--static")
 			static=1
 			;;
-		"--with-claspre")
-			claspre=1
-			;;
 		"--with-mt")
 			mt=1
 			;;
@@ -82,16 +78,15 @@ while [[ $# > 0 ]]; do
 			echo "    Default: '/usr/local'"
 			echo "  --bindir=PATH      : set install path to PATH"
 			echo "    Default: '${PREFIX}/bin'"
-			echo 
+			echo
 			echo "  --config=NAME      : set configuration to NAME"
 			echo "    NAME=release     : configure for optimized release version"
 			echo "    NAME=debug       : configure for debug version"
 			echo "    NAME=check       : configure for release version with assertions enabled"
 			echo "    <NAME>           : configure for custom configuration with name <NAME>"
-			echo 
+			echo
 			echo "  --with-mt          : enable multi-thread support (see below)"
 			echo "  --set-rpath        : store path to shared libraries in binary header"
-			echo "  --with-claspre     : enable claspre features in clasp"
 			echo "  --static           : link statically (if supported)"
 			echo "  --m32              : force 32-bit binary (if supported)"
 			echo "  --m64              : force 64-bit binary (if supported)"
@@ -133,25 +128,25 @@ BUILDPATH="build/${CONFIG}"
 
 if [[ $mt == 0 ]]; then
 	CXXFLAGS="${CXXFLAGS} -DWITH_THREADS=0"
-else	
+else
 	# try to find tbb headers
 	echo -ne "Checking for TBB include path..."
 	for i in "$TBB_INCLUDE" "$TBB30_INSTALL_DIR/include" "/opt/intel/tbb/include" "/usr/local/include" "/usr/include" "/opt/local/include"; do
 		TBB_INCLUDE=""
 		if [ -f "$i/tbb/tbb.h" ]; then
-			TBB_INCLUDE="$i"				
+			TBB_INCLUDE="$i"
 			echo "$TBB_INCLUDE"
 			break
 		fi
 	done
 	if [ -z "$TBB_INCLUDE" ]; then
-		echo "FAIL"		
+		echo "FAIL"
 		echo "*** Error: TBB include path not set!"
 		echo "use '$0 TBB_INCLUDE=<path_to_tbb_include>'"
 		exit 1
 	fi
 	# try to find tbb lib
-	echo -ne "Checking for TBB library path..."	
+	echo -ne "Checking for TBB library path..."
 	for i in "$TBB_LIB" "$TBB30_INSTALL_DIR/lib" "/opt/intel/tbb/lib" "/usr/local/lib" "/usr/lib" "/opt/local/lib" ; do
 		TBB_LIB=""
 		if [ -f "$i/libtbb.so" -o -f "$i/libtbb.dylib" ]; then
@@ -161,21 +156,21 @@ else
 		fi
 	done
 	if [ -z "$TBB_LIB" ]; then
-		echo "FAIL"		
+		echo "FAIL"
 		echo "*** Error: TBB library path not set or 'libtbb.{so,dylib}' not found!"
 		echo "use '$0 TBB_LIB=<path_to_tbb_library>'"
 		exit 1
 	fi
 	CXXFLAGS="${CXXFLAGS} -DWITH_THREADS=1 -I\"${TBB_INCLUDE}\""
 	LDFLAGS="${LDFLAGS} -L\"${TBB_LIB}\""
-	LDLIBS="-ltbb" 
+	LDLIBS="-ltbb"
 	if [[ $rpath == 1 ]]; then
-		LDFLAGS="${LDFLAGS} -Xlinker \"--rpath=\"${TBB_LIB}\"\""
+		LDFLAGS="${LDFLAGS} -Xlinker -rpath -Xlinker \"${TBB_LIB}\""
 	fi
 	BUILDPATH="${BUILDPATH}_mt"
 fi
 
-CXXFLAGS="${CXXFLAGS} -DWITH_CLASPRE=${claspre}"
+CXXFLAGS="${CXXFLAGS}"
 if [[ $static == 1 ]]; then
 	LDFLAGS="${LDFLAGS} -static"
 	BUILDPATH="${BUILDPATH}_static"
@@ -193,16 +188,21 @@ fi
 
 # create & prepare build hierarchy
 ROOTPATH="../.."
-LIB_CLASP="libclasp"
-LIB_OPTS="libprogram_opts"
+LIBS="libclasp libprogram_opts"
+LIB_TARGETS=""
+LIB_INCLUDES=""
 mkdir -p "$BUILDPATH/app"
 mkdir -p "$BUILDPATH/bin"
-mkdir -p "$BUILDPATH/$LIB_CLASP/lib"
-mkdir -p "$BUILDPATH/$LIB_OPTS/lib"
+for lib in $LIBS; do 
+	mkdir -p "$BUILDPATH/$lib/lib" 
+	LIB_TARGETS="${LIB_TARGETS} ${lib}/lib/${lib}.a"
+	LIB_INCLUDES="-I\$(PROJECT_ROOT)/${lib} ${LIB_INCLUDES}"
+done
 cd "$BUILDPATH"
-rm -f  .CONFIG $LIB_CLASP/.CONFIG $LIB_OPTS/.CONFIG
-rm -f  Makefile $LIB_CLASP/Makefile $LIB_OPTS/Makefile
-rm -f FLAGS
+rm -f .CONFIG Makefile FLAGS
+for lib in $LIBS; do 
+	rm -f $lib/.CONFIG $lib/Makefile
+done
 # write FLAGS
 touch FLAGS
 if [ ! -z "$CXX" ]; then
@@ -214,12 +214,12 @@ echo "CXXFLAGS    := ${CXXFLAGS}" >> FLAGS
 echo "WARNFLAGS   := -W -Wall"    >> FLAGS
 echo "LDFLAGS     := ${LDFLAGS}"  >> FLAGS
 echo ""                           >> FLAGS
-
 # create Makefiles
 LIB_MAKES="${ROOTPATH}/tools/Base.in ${ROOTPATH}/tools/LibRule.in ${ROOTPATH}/tools/BaseRule.in"
 PRO_MAKES="${ROOTPATH}/tools/Base.in ${ROOTPATH}/tools/ProjRule.in ${ROOTPATH}/tools/BaseRule.in"
-cat $LIB_MAKES >> $LIB_CLASP/Makefile
-cat $LIB_MAKES >> $LIB_OPTS/Makefile
+for lib in $LIBS; do 
+	cat $LIB_MAKES >> $lib/Makefile
+done
 cat $PRO_MAKES >> Makefile
 # write project config
 touch  .CONFIG
@@ -229,9 +229,9 @@ echo "FLAGS        := FLAGS"                >> .CONFIG
 echo "SOURCE_DIR   := \$(PROJECT_ROOT)/app" >> .CONFIG
 echo "INCLUDE_DIR  := \$(PROJECT_ROOT)/app" >> .CONFIG
 echo "OUT_DIR      := app"                  >> .CONFIG
-echo "INCLUDES     := -I\$(PROJECT_ROOT)/${LIB_CLASP} -I\$(PROJECT_ROOT)/${LIB_OPTS}" >> .CONFIG
-echo "SUBDIRS      := ${LIB_CLASP} ${LIB_OPTS}"  >> .CONFIG
-echo "LIBS         := ${LIB_CLASP}/lib/${LIB_CLASP}.a ${LIB_OPTS}/lib/${LIB_OPTS}.a" >> .CONFIG
+echo "INCLUDES     := ${LIB_INCLUDES}"      >> .CONFIG
+echo "SUBDIRS      := ${LIBS}"              >> .CONFIG
+echo "LIBS         := ${LIB_TARGETS:1}"     >> .CONFIG
 echo "LDLIBS       := ${LDLIBS}"            >> .CONFIG
 echo "INSTALL_DIR  := \"${INSTALLPATH}\""   >> .CONFIG
 if [ ! -z "$POST_BUILD" ]; then
@@ -239,24 +239,19 @@ echo "POST_BUILD  := $POST_BUILD"           >> .CONFIG
 fi
 echo ""                                     >> .CONFIG
 # write lib configs
-touch $LIB_CLASP/.CONFIG $LIB_OPTS/.CONFIG
-echo "PROJECT_ROOT := ${ROOTPATH}/.."      >> $LIB_CLASP/.CONFIG
-echo "PROJECT_ROOT := ${ROOTPATH}/.."      >> $LIB_OPTS/.CONFIG
-echo "TARGET       := lib/${LIB_CLASP}.a"  >> $LIB_CLASP/.CONFIG
-echo "TARGET       := lib/${LIB_OPTS}.a"   >> $LIB_OPTS/.CONFIG
-echo "FLAGS        := ../FLAGS"            >> $LIB_CLASP/.CONFIG
-echo "FLAGS        := ../FLAGS"            >> $LIB_OPTS/.CONFIG
-echo "SOURCE_DIR   := \$(PROJECT_ROOT)/${LIB_CLASP}/src" >> $LIB_CLASP/.CONFIG
-echo "TEST_DIR     := \$(PROJECT_ROOT)/${LIB_CLASP}/tests" >> $LIB_CLASP/.CONFIG
-echo "TEST_TARGET  := ./test-lib"          >> $LIB_CLASP/.CONFIG
-echo "SOURCE_DIR   := \$(PROJECT_ROOT)/${LIB_OPTS}/src" >> $LIB_OPTS/.CONFIG
-echo "INCLUDE_DIR  := \$(PROJECT_ROOT)/${LIB_CLASP}/clasp" >> $LIB_CLASP/.CONFIG
-echo "INCLUDE_DIR  := \$(PROJECT_ROOT)/${LIB_OPTS}/program_opts" >> $LIB_OPTS/.CONFIG
-echo "INCLUDES     := -I\$(PROJECT_ROOT)/${LIB_CLASP}" >> $LIB_CLASP/.CONFIG
-echo "INCLUDES     := -I\$(PROJECT_ROOT)/${LIB_OPTS}" >> $LIB_OPTS/.CONFIG
-echo "" >> $LIB_CLASP/.CONFIG
-echo "" >> $LIB_OPTS/.CONFIG
-
+for lib in $LIBS; do
+	touch $lib/.CONFIG
+	echo "PROJECT_ROOT := ${ROOTPATH}/.."      >> $lib/.CONFIG
+	echo "TARGET       := lib/${lib}.a"        >> $lib/.CONFIG
+	echo "FLAGS        := ../FLAGS"            >> $lib/.CONFIG
+	echo "TEST_DIR     := \$(PROJECT_ROOT)/${lib}/tests" >> $lib/.CONFIG
+	echo "TEST_TARGET  := ./test-lib"          >> $lib/.CONFIG
+	echo "LDLIBS       := ${LDLIBS}"           >> $lib/.CONFIG
+	echo "SOURCE_DIR   := \$(PROJECT_ROOT)/${lib}/src" >> $lib/.CONFIG
+	echo "INCLUDE_DIR  := \$(PROJECT_ROOT)/${lib}/${lib:3}" >> $lib/.CONFIG
+	echo "INCLUDES     := ${LIB_INCLUDES}" >> $lib/.CONFIG
+	echo "" >> $lib/.CONFIG
+done
 # DONE
 echo
 echo "Configuration successfully written to ${BUILDPATH}."
@@ -270,11 +265,11 @@ echo "To install clasp afterwards type:"
 echo "  make install"
 echo "or copy '${BUILDPATH}/clasp' to a directory of your choice."
 if [ ! -d "$INSTALLPATH" ]; then
-echo 
+echo
 echo "Note: install path '$INSTALLPATH' does not exist"
 echo "      'make install' will fail unless it is first created!"
 fi
-echo 
+echo
 echo "Note: \"make\" must correspond to GNU Make 3.8 or later."
 echo
 
