@@ -72,6 +72,7 @@ inline RuleType& operator++(RuleType& x) {
 		default:             return x = BASICRULE;
 	}
 }
+inline bool hasWeights(RuleType t) { return t == WEIGHTRULE || t == OPTIMIZERULE; }
 
 //! Supported edge types.
 /*!
@@ -294,9 +295,9 @@ public:
 	void  clearBody(Literal p)            { clear(p.var(), pos_flag+p.sign()); }
 	void  clearAll()                      { StateVec().swap(state_); }
 
-	bool  allMarked(const VarVec& vec, uint8 f) const {
-		for (VarVec::const_iterator it = vec.begin(), end = vec.end(); it != end; ++it) {
-			if (!isSet(*it, f)) return false;
+	bool  allMarked(EdgeIterator first, EdgeIterator last, uint8 f) const {
+		for (; first != last; ++first) {
+			if (!isSet(first->node(), f)) return false;
 		}
 		return true;
 	}
@@ -557,6 +558,7 @@ public:
 	bool     simplifyHeads(LogicProgram& prg, bool strong);
 	bool     mergeHeads(LogicProgram& prg, PrgBody& heads, bool strong, bool simplify = true);
 	void     removeHead(PrgHead* h, EdgeType t);
+	bool     hasHead(PrgHead* h, EdgeType t) const;
 	//! Simplifies the body, i.e. its predecessors-lists.
 	/*!
 	 * - removes true/false atoms from B+/B- resp.
