@@ -30,8 +30,6 @@
 
 namespace Clasp { namespace SatElite {
 
-class Clause; // Special clause class optimized for use with the preprocessor
-
 //! SatElite preprocessor for clauses
 /*!
  * The preprocessor implements subsumption, self-subsumption, variable elimination,
@@ -45,33 +43,24 @@ class Clause; // Special clause class optimized for use with the preprocessor
  */
 class SatElite : public Clasp::SatPreprocessor {
 public:
-	explicit SatElite(SharedContext* ctx = 0);
+	SatElite();
 	~SatElite();
-
-	//! adds a clause to the preprocessor
-	/*!
-	 * \pre clause is not a tautology (i.e. does not contain both l and ~l)
-	 * \pre clause is a set (i.e. does not contain l more than once)
-	 * \return true if clause was added. False if adding the clause makes the problem UNSAT
-	 */
-	bool addClause(const LitVec& clause);
-
-	//! Options used by the preprocessor
-	struct Options {
-		Options() : maxFrozen(1.0), maxTime(UINT32_MAX), maxIters(UINT32_MAX), maxOcc(UINT32_MAX), maxClause(4000000), bce(1), elimPure(true) {}
-		double  maxFrozen;/**< run only if #frozen/#vars <= maxFrozen           */
-		uint32  maxTime;  /**< maximal runtime, checked after each iteration    */
-		uint32  maxIters; /**< maximal number of iterations                     */
-		uint32  maxOcc;   /**< skip v, if #occ(v) > maxOcc && #occ(~v) > maxOcc */
-		uint32  maxClause;/**< run only if #clauses < maxClause                 */
-		uint32  bce;      /**< blocked clause elimination (0=off, 1=limited, 2=full */
-		bool    elimPure; /**< eliminate pure literals?                         */
-	} options;
-
-	bool preprocess(bool enumModels);
-	void extendModel( Assignment& m, LitVec& unconstr );
-	bool limit(uint32 numCons) const { return numCons > options.maxClause; }
-	typedef PodVector<Clause*>::type  ClauseList;
+	Clasp::SatPreprocessor* clone();
+	struct Progress : public Event_t<Progress> {
+		enum EventOp { event_algorithm = '*', event_bce = 'B', event_var_elim = 'E', event_subsumption = 'S', };
+		Progress(SatElite* p, EventOp o, uint32 i, uint32 m) : Event_t<Progress>(Event::subsystem_prepare, Event::verbosity_high), self(p), cur(i), max(m) {
+			op = (uint32)o;
+		}
+		SatElite* self;
+		uint32    cur;
+		uint32    max;
+	};
+protected:
+	bool  initPreprocess(Options& opts);
+	void  reportProgress(Progress::EventOp, uint32 curr, uint32 max);
+	bool  doPreprocess();
+	void  doExtendModel(ValueVec& m, LitVec& open);
+	void  doCleanUp();
 private:
 	typedef PodVector<uint8>::type    TouchedList;
 	typedef bk_lib::left_right_sequence<Literal, Var,0> ClWList;
@@ -113,8 +102,8 @@ private:
 			else { dirty = 1; }
 		}
 		// note: only one literal of v shall be marked at a time
-		bool    marked(bool sign) const   { return (litMark & (1+sign)) != 0; }
-		void    mark(bool sign)           { litMark = (1+sign); }
+		bool    marked(bool sign) const   { return (litMark & (1+int(sign))) != 0; }
+		void    mark(bool sign)           { litMark = (1+int(sign)); }
 		void    unmark()                  { litMark = 0; }
 	};
 	struct LessOccCost {
@@ -127,13 +116,13 @@ private:
 	typedef bk_lib::indexed_priority_queue<LessOccCost> ElimHeap;
 	Clause*         peekSubQueue() const {
 		assert(qFront_ < queue_.size());
-		return clauses_[ queue_[qFront_] ];
+		return (Clause*)clause( queue_[qFront_] );
 	}
 	inline Clause*  popSubQueue();
 	inline void     addToSubQueue(uint32 clauseId);
 	void            updateHeap(Var v) {
 		assert(ctx_);
-		if (!ctx_->frozen(v) && !ctx_->eliminated(v)) {
+		if (!ctx_->varInfo(v).frozen() && !ctx_->eliminated(v)) {
 			elimHeap_.update(v);
 			if (occurs_[v].bce == 0 && occurs_[0].bce != 0) {
 				occurs_[0].addWatch(v);
@@ -159,15 +148,13 @@ private:
 	void    unmarkAll(const Literal* lits, uint32 size) const;
 	bool    addResolvent(uint32 newId, const Clause& c1, const Clause& c2);
 	bool    cutoff(Var v) const {
-		return (occurs_[v].pos > options.maxOcc && occurs_[v].neg > options.maxOcc)
-			||   (occurs_[v].cost() == 0 && options.elimPure == 0);
+		return opts_->occLimit(occurs_[v].pos, occurs_[v].neg)
+		  ||   (occurs_[v].cost() == 0 && opts_->mode == Options::prepro_preserve_models);
 	}
-	void    cleanUp();
 	bool    timeout() const { return time(0) > timeout_; }
+	const Options*opts_;      // active options
 	OccurList*    occurs_;    // occur list for each variable
-	Clause*       elimTop_;   // stack of blocked/eliminated clauses
 	ElimHeap      elimHeap_;  // candidates for variable elimination; ordered by increasing occurrence-cost
-	ClauseList    clauses_;   // all clauses
 	VarVec      posT_, negT_; // temporary clause lists used in eliminateVar
 	ClauseList    resCands_;  // pairs of clauses to be resolved
 	LitVec        resolvent_; // temporary, used in addResolvent
