@@ -26,15 +26,21 @@
 #define WIN32_LEAN_AND_MEAN // exclude APIs such as Cryptography, DDE, RPC, Shell, and Windows Sockets.
 #define NOMINMAX            // do not let windows.h define macros min and max
 #endif
-#include <tbb/compat/thread> // replace with std::thread once available
+#include <tbb/tbb_thread.h>
+namespace Clasp {
+	typedef tbb::tbb_thread thread;
+	namespace this_thread {
+		using tbb::this_tbb_thread::yield;
+	}
+}
 #else
-namespace Clasp { namespace Serial {
+namespace no_multi_threading {
 	struct thread {};
 	inline void yield() {}
-}}
-namespace std { 
-	using Clasp::Serial::thread;
-	namespace this_thread { using Clasp::Serial::yield; }
+}
+namespace Clasp { 
+	typedef no_multi_threading::thread thread;
+	namespace this_thread { using no_multi_threading::yield; }
 }
 #endif
 
