@@ -24,104 +24,32 @@
 #pragma warning (disable : 4200) // nonstandard extension used : zero-sized array
 #endif
 namespace Clasp { namespace SatElite {
-
-inline uint64 abstractLit(Literal p) {
-	assert(p.var() > 0); 
-	return uint64(1) << ((p.var()-1) & 63); 
-}
-
-// A clause class optimized for the SatElite preprocessor.
-// Uses 1-literal watching to implement forward-subsumption
-class Clause {
-public:
-	static Clause* newClause(const LitVec& lits) {
-		void* mem = ::operator new( sizeof(Clause) + (lits.size()*sizeof(Literal)) );
-		return new (mem) Clause(lits);
-	}
-	bool simplify(Solver& s) {
-		uint32 i;
-		for (i = 0; i != size_ && s.value(lits_[i].var()) == value_free; ++i) {;}
-		if      (i == size_)          { return false; }
-		else if (s.isTrue(lits_[i]))  { return true;  }
-		uint32 j = i++;
-		for (; i != size_; ++i) {
-			if (s.isTrue(lits_[i]))   { return true; }
-			if (!s.isFalse(lits_[i])) { lits_[j++] = lits_[i]; }
-		}
-		size_ = j;
-		return false;
-	}
-	void destroy() {
-		void* mem = this;
-		this->~Clause();
-		::operator delete(mem);
-	}
-	uint32          size()                const { return size_; }
-	const Literal&  operator[](uint32 x)  const { return lits_[x]; }
-	bool            inSQ()                const { return inSQ_ != 0; }
-	uint64          abstraction()         const { return data_.abstr; }
-	Clause*         next()                const { return data_.next;  }
-	bool            blocked()             const { return inSQ_   != 0;  }
-	bool            marked()              const { return marked_ != 0; }
-	Literal&        operator[](uint32 x)        { return lits_[x]; }    
-	void            setInSQ(bool b)             { inSQ_   = (uint32)b; }
-	void            setMarked(bool b)           { marked_ = (uint32)b; }
-	uint64&         abstraction()               { return data_.abstr; }
-	void            strengthen(Literal p)       {
-		uint64 a = 0;
-		uint32 i, end;
-		for (i   = 0; lits_[i] != p; ++i) { a |= abstractLit(lits_[i]); }
-		for (end = size_-1; i < end; ++i) { lits_[i] = lits_[i+1]; a |= abstractLit(lits_[i]); }
-		--size_;
-		data_.abstr = a;
-	}
-	Clause*         linkRemoved(bool blocked, Clause* next) {
-		inSQ_      = blocked;
-		data_.next = next;
-		return this;
-	}
-private:
-	Clause(const LitVec& lits) : size_((uint32)lits.size()), inSQ_(0), marked_(0) {
-		std::memcpy(lits_, &lits[0], lits.size()*sizeof(Literal));
-	}
-	union {
-		uint64  abstr;      // abstraction - to speed up backward subsumption check
-		Clause* next;       // next removed clause
-	}       data_;
-	uint32  size_   : 30; // size of the clause
-	uint32  inSQ_   : 1;  // in subsumption-queue?
-	uint32  marked_ : 1;  // a marker flag
-	Literal lits_[0];     // literals of the clause: [lits_[0], lits_[size_])
-};
 /////////////////////////////////////////////////////////////////////////////////////////
 // SatElite preprocessing
 //
 /////////////////////////////////////////////////////////////////////////////////////////
-SatElite::SatElite(SharedContext* ctx) 
+SatElite::SatElite() 
 	: occurs_(0)
-	, elimTop_(0)
 	, elimHeap_(LessOccCost(occurs_))
 	, qFront_(0)
 	, facts_(0) {
-	if (ctx) setContext(*ctx);
 }
 
 SatElite::~SatElite() {
-	cleanUp();
-	for (Clause* r = elimTop_; r;) {
-		Clause* t = r;
-		 r = r->next();
-		 t->destroy();
-	}
-	elimTop_ = 0;
+	SatElite::doCleanUp();
 }
 
-void SatElite::cleanUp() {
+void SatElite::reportProgress(Progress::EventOp id, uint32 curr, uint32 max) {
+	ctx_->report(Progress(this, id, curr, max));
+}
+
+Clasp::SatPreprocessor* SatElite::clone() {
+	SatElite* cp = new SatElite();
+	return cp;
+}
+
+void SatElite::doCleanUp() {
 	delete [] occurs_;  occurs_ = 0; 
-	for (ClauseList::size_type i = 0; i != clauses_.size(); ++i) {
-		if (clauses_[i]) { clauses_[i]->destroy(); }
-	}
-	ClauseList().swap(clauses_);
 	ClauseList().swap(resCands_);
 	VarVec().swap(posT_);
 	VarVec().swap(negT_);
@@ -131,45 +59,30 @@ void SatElite::cleanUp() {
 	qFront_ = facts_ = 0;
 }
 
-bool SatElite::addClause(const LitVec& clause) {
-	assert(ctx_ && ctx_->master());
-	if (clause.empty()) {
-		return false;
-	}
-	else if (clause.size() == 1) {
-		return ctx_->master()->force(clause[0], 0) && ctx_->master()->propagate();
-	}
-	else {
-		Clause* c = Clause::newClause( clause );
-		clauses_.push_back( c );
-	}
-	return true;
-}
-
-Clause* SatElite::popSubQueue() {
-	if (Clause* c = clauses_[ queue_[qFront_++] ]) {
-		c->setInSQ(false);
+SatPreprocessor::Clause* SatElite::popSubQueue() {
+	if (Clause* c = clause( queue_[qFront_++] )) {
+		c->setInQ(false);
 		return c;
 	}
 	return 0;
 }
 
 void SatElite::addToSubQueue(uint32 clauseId) {
-	assert(clauses_[clauseId] != 0);
-	if (!clauses_[clauseId]->inSQ()) {
+	assert(clause(clauseId) != 0);
+	if (!clause(clauseId)->inQ()) {
 		queue_.push_back(clauseId);
-		clauses_[clauseId]->setInSQ(true);
+		clause(clauseId)->setInQ(true);
 	}
 }
 
 void SatElite::attach(uint32 clauseId, bool initialClause) {
-	Clause& c = *clauses_[clauseId];
+	Clause& c = *clause(clauseId);
 	c.abstraction() = 0;
 	for (uint32 i = 0; i != c.size(); ++i) {
 		Var v = c[i].var();
 		occurs_[v].add(clauseId, c[i].sign());
 		occurs_[v].unmark();
-		c.abstraction() |= abstractLit(c[i]);
+		c.abstraction() |= Clause::abstractLit(c[i]);
 		if (elimHeap_.is_in_queue(v)) {
 			elimHeap_.decrease(v);
 		}
@@ -183,20 +96,18 @@ void SatElite::attach(uint32 clauseId, bool initialClause) {
 }
 
 void SatElite::detach(uint32 id) {
-	Clause& c  = *clauses_[id];
+	Clause& c  = *clause(id);
 	occurs_[c[0].var()].removeWatch(id);
 	for (uint32 i = 0; i != c.size(); ++i) {
 		Var v = c[i].var();
 		occurs_[v].remove(id, c[i].sign(), false);
 		updateHeap(v);
 	}
-	clauses_[id] = 0;
-	c.destroy();
-	++stats.clRemoved;
+	destroyClause(id);
 }
 
 void SatElite::bceVeRemove(uint32 id, bool freeId, Var ev, bool blocked) {
-	Clause& c  = *clauses_[id];
+	Clause& c  = *clause(id);
 	occurs_[c[0].var()].removeWatch(id);
 	uint32 pos = 0;
 	for (uint32 i = 0; i != c.size(); ++i) {
@@ -210,90 +121,33 @@ void SatElite::bceVeRemove(uint32 id, bool freeId, Var ev, bool blocked) {
 			pos = i;
 		}
 	}
-	clauses_[id] = 0;
 	std::swap(c[0], c[pos]);
-	elimTop_   = c.linkRemoved(blocked, elimTop_);
-	++stats.clRemoved;
+	c.setMarked(blocked);
+	eliminateClause(id);
 }
 
-bool SatElite::preprocess(bool enumModels) {
-	struct Scope { SatElite* self; char type; ~Scope() { 
-		self->cleanUp();
-		self->reportProgress(Event(self, type, 100,100));
-	}} sc = {this, '*'};
-	Solver* s = ctx_->master();
-	// skip preprocessing if other constraints are UNSAT
-	if (!s->propagate()) return false;
-	// preprocess only if not too many vars are frozen
-	double frozen = 0.0;
-	double maxF   = std::min(1.0, options.maxFrozen/100.0);
-	if (maxF != 1.0 && (frozen = ctx_->stats().vars_frozen) != 0.0) {
-		for (LitVec::const_iterator it = s->trail().begin(), end = s->trail().end(); it != end; ++it) {
-			frozen -= (ctx_->frozen(it->var()));
-		}
-		frozen = std::min(1.0,  frozen / double(s->numFreeVars()));
+bool SatElite::initPreprocess(Options& opts) {
+	reportProgress(Progress::event_algorithm, 0,100);
+	opts_     = &opts;
+	occurs_   = new OccurList[ctx_->numVars()+1];
+	qFront_   = 0;
+	occurs_[0].bce = (opts.type == Options::sat_pre_full);
+	return true;
+}
+bool SatElite::doPreprocess() {
+	// 1. add clauses to occur lists
+	for (uint32 i  = 0, end = numClauses(); i != end; ++i) {
+		attach(i, true);
 	}
-	if (!SatElite::limit((uint32)clauses_.size()) && frozen <= maxF) {
-		// 0. allocate & init state
-		occurs_   = new OccurList[ctx_->numVars()+1];
-		qFront_   = 0;
-		if (enumModels) {
-			options.elimPure = 0;
-			options.bce      = 0;
-		}
-		occurs_[0].bce = (options.bce > 1);
-		// 1. remove SAT-clauses, strengthen clauses w.r.t false literals, init occur-lists
-		facts_ = s->numAssignedVars();
-		ClauseList::size_type j = 0; 
-		for (ClauseList::size_type i = 0; i != clauses_.size(); ++i) {
-			Clause* c = clauses_[i]; assert(c);
-			if      (c->simplify(*s)) { c->destroy(); clauses_[i] = 0;}
-			else if (c->size() < 2)         {
-				Literal unit = c->size() == 1 ? (*c)[0] : negLit(0);
-				c->destroy(); clauses_[i] = 0;
-				if (!(s->force(unit, 0)&&s->propagate()) || !propagateFacts()) {
-					for (++i; i != clauses_.size(); ++i) {
-						clauses_[i]->destroy();
-					}
-					clauses_.erase(clauses_.begin()+j, clauses_.end());
-					return false;
-				}
-			}
-			else                            {
-				clauses_[j] = c;
-				attach(uint32(j), true);
-				++j;
-			}
-		}
-		clauses_.erase(clauses_.begin()+j, clauses_.end());
-		assert(facts_ == s->numAssignedVars());
-		// simplify other constraints w.r.t new derived top-level facts
-		if (!s->simplify()) return false;
-		// 2. remove subsumed clauses, eliminate vars by clause distribution
-		timeout_ = options.maxTime != UINT32_MAX ? time(0) + options.maxTime : std::numeric_limits<std::time_t>::max();
-		for (uint32 i = 0, end = options.maxIters; queue_.size()+elimHeap_.size() > 0; ++i) {
-			if (!backwardSubsume())          { return false; }
-			if (timeout() || i == end)       { break; }
-			if (!eliminateVars())            { return false; }
-		}
-		assert( facts_ == s->numAssignedVars() );
+	// 2. remove subsumed clauses, eliminate vars by clause distribution
+	timeout_ = opts_->limTime ? time(0) + opts_->limTime : std::numeric_limits<std::time_t>::max();
+	for (uint32 i = 0, end = opts_->limIters ? opts_->limIters : UINT32_MAX; queue_.size()+elimHeap_.size() > 0; ++i) {
+		if (!backwardSubsume())   { return false; }
+		if (timeout() || i == end){ break;        }
+		if (!eliminateVars())     { return false; }
 	}
-	sc.type = '*';
-	// simplify other constraints w.r.t new derived top-level facts
-	if (!s->simplify()) return false;
-	// 3. Transfer simplified clausal problem to solver
-	ClauseCreator nc(s);
-	for (ClauseList::size_type i = 0; i != clauses_.size(); ++i) {
-		if (clauses_[i]) {
-			Clause& c = *clauses_[i];
-			nc.start();
-			for (uint32 x = 0; x != c.size(); ++x) {  nc.add(c[x]);  }
-			c.destroy();
-			nc.end();
-		}
-	}
-	clauses_.clear();
-	return true;  
+	reportProgress(Progress::event_algorithm, 100,100);
+	return true;
 }
 
 // (Destructive) unit propagation on clauses.
@@ -308,7 +162,7 @@ bool SatElite::propagateFacts() {
 		OccurList& ov = occurs_[l.var()];
 		ClRange cls   = occurs_[l.var()].clauseRange();
 		for (ClIter x = cls.first; x != cls.second; ++x) {
-			if      (clauses_[x->var()] == 0)        { continue; }
+			if      (clause(x->var()) == 0)          { continue; }
 			else if (x->sign() == l.sign())          { detach(x->var()); }
 			else if (!strengthenClause(x->var(), ~l)){ return false; }
 		}
@@ -325,7 +179,7 @@ bool SatElite::backwardSubsume() {
 	while (qFront_ != queue_.size()) {
 		if ((qFront_ & 8191) == 0) {
 			if (timeout()) break;
-			if (queue_.size() > 1000) reportProgress(Event(this, 'S', qFront_, queue_.size()));
+			if (queue_.size() > 1000) reportProgress(Progress::event_subsumption, qFront_, queue_.size());
 		}
 		if (peekSubQueue() == 0) { ++qFront_; continue; }
 		Clause& c = *popSubQueue();
@@ -344,7 +198,7 @@ bool SatElite::backwardSubsume() {
 		for (uint32 i = 0, end = cls.left_size(); i != end; ++i) {
 			Literal cl      = cls.left(i);
 			uint32 otherId  = cl.var();
-			Clause* other   = clauses_[otherId];
+			Clause* other   = clause(otherId);
 			if (other && other!= &c && (res = subsumes(c, *other, best.sign()==cl.sign()?posLit(0):best)) != negLit(0)) {
 				if (res == posLit(0)) {
 					// other is subsumed - remove it
@@ -358,7 +212,7 @@ bool SatElite::backwardSubsume() {
 					occurs_[res.var()].remove(otherId, res.sign(), res.var() != best.var());
 					updateHeap(res.var());
 					if (!strengthenClause(otherId, res))              { return false; }
-					if (res.var() == best.var() || clauses_[otherId] == 0)  { other = 0; }
+					if (res.var() == best.var() || !clause(otherId))  { other = 0; }
 				}
 			}
 			if (other && j++ != i)  { cls.left(j-1) = cl; }
@@ -435,7 +289,7 @@ bool SatElite::subsumed(LitVec& cl) {
 		ClWList& cls   = occurs_[l.var()].refs; // right: all clauses watching either l or ~l
 		WIter wj       = cls.right_begin();
 		for (WIter w = wj, end = cls.right_end(); w != end; ++w) {
-			Clause& c = *clauses_[*w];
+			Clause& c = *clause(*w);
 			if (c[0] == l)  {
 				if ( (x = findUnmarkedLit(c, 1)) == c.size() ) {
 					while (w != end) { *wj++ = *w++; }
@@ -479,7 +333,7 @@ removeLit:;
 // Pre: c contains l
 // Pre: c was already removed from l's occur-list
 bool SatElite::strengthenClause(uint32 clauseId, Literal l) {
-	Clause& c = *clauses_[clauseId];
+	Clause& c = *clause(clauseId);
 	if (c[0] == l) {
 		occurs_[c[0].var()].removeWatch(clauseId);
 		// Note: Clause::strengthen shifts literals after l to the left. Thus
@@ -505,7 +359,7 @@ SatElite::ClRange SatElite::splitOcc(Var v, bool mark) {
 	posT_.clear(); negT_.clear();
 	ClIter j = cls.first;
 	for (ClIter x = j; x != cls.second; ++x) {
-		if (Clause* c = clauses_[x->var()]) {
+		if (Clause* c = clause(x->var())) {
 			assert(c->marked() == false);
 			c->setMarked(mark);
 			(x->sign() ? negT_ : posT_).push_back(x->var());
@@ -535,23 +389,23 @@ void SatElite::unmarkAll(const Literal* lits, uint32 size) const {
 bool SatElite::bceVe(Var v, uint32 maxCnt) {
 	Solver* s = ctx_->master();
 	if (s->value(v) != value_free) return true;
-	assert(!ctx_->frozen(v) && !ctx_->eliminated(v));
+	assert(!ctx_->varInfo(v).frozen() && !ctx_->eliminated(v));
 	resCands_.clear();
 	// distribute clauses on v 
 	// check if number of clauses decreases if we'd eliminate v
-	uint32  bce    = options.bce;
+	uint32 bce     = opts_->bce();
 	ClRange cls    = splitOcc(v, bce > 1);
-	uint32  cnt    = 0;
-	uint32  markMax= ((uint32)negT_.size() * (bce>1));
-	uint32  blocked= 0;
+	uint32 cnt     = 0;
+	uint32 markMax = ((uint32)negT_.size() * (bce>1));
+	uint32 blocked = 0;
 	bool stop      = false;
 	Clause* lhs, *rhs;
 	for (VarVec::const_iterator i = posT_.begin(); i != posT_.end() && !stop; ++i) {
-		lhs         = clauses_[*i];
+		lhs         = clause(*i);
 		markAll(&(*lhs)[0], lhs->size());
 		lhs->setMarked(bce != 0);
 		for (VarVec::const_iterator j = negT_.begin(); j != negT_.end(); ++j) {
-			if (!trivialResolvent(*(rhs = clauses_[*j]), v)) {
+			if (!trivialResolvent(*(rhs = clause(*j)), v)) {
 				markMax -= rhs->marked();
 				rhs->setMarked(false); // not blocked on v
 				lhs->setMarked(false); // not blocked on v
@@ -577,7 +431,7 @@ bool SatElite::bceVe(Var v, uint32 maxCnt) {
 		// (partial) models can be extended.
 		for (ClIter it = cls.first; it != cls.second; ++it) {
 			// reuse first cnt ids for resolvents
-			if (clauses_[it->var()]) {
+			if (clause(it->var())) {
 				bool freeId = (cnt && cnt--);
 				bceVeRemove(it->var(), freeId, v, false);
 			}
@@ -600,13 +454,13 @@ bool SatElite::bceVe(Var v, uint32 maxCnt) {
 			bceVeRemove(posT_[i], false, v, true);
 		}
 		for (VarVec::const_iterator it = negT_.begin(); markMax; ++it) {
-			if ( (rhs = clauses_[*it])->marked() ) {
+			if ( (rhs = clause(*it))->marked() ) {
 				bceVeRemove(*it, false, v, true);
 				--markMax;
 			}
 		}
 	}
-	return options.maxIters != UINT32_MAX || backwardSubsume();
+	return opts_->limIters != 0 || backwardSubsume();
 }
 
 bool SatElite::bce() {
@@ -617,7 +471,7 @@ bool SatElite::bce() {
 		occurs_[v].bce=0; 
 		if ((ops & 1023) == 0)   {
 			if (timeout())         { bce.clear(); return true; }
-			if ((ops & 8191) == 0) { reportProgress(Event(this, 'B', ops, 1+bce.size())); }
+			if ((ops & 8191) == 0) { reportProgress(Progress::event_bce, ops, 1+bce.size()); }
 		}
 		if (!cutoff(v) && !bceVe(v, 0)) { return false; }
 	}
@@ -633,13 +487,13 @@ bool SatElite::eliminateVars() {
 		occ = occurs_[v].numOcc();
 		if ((ops & 1023) == 0)   {
 			if (timeout())         { elimHeap_.clear(); return true; }
-			if ((ops & 8191) == 0) { reportProgress(Event(this, 'E', ops, 1+elimHeap_.size())); }
+			if ((ops & 8191) == 0) { reportProgress(Progress::event_var_elim, ops, 1+elimHeap_.size()); }
 		}
 		if (!cutoff(v) && !bceVe(v, occ)) {
 			return false;
 		}
 	}
-	return options.maxIters != UINT32_MAX || bce();
+	return opts_->limIters != 0 || bce();
 }
 
 // returns true if the result of resolving c1 (implicitly given) and c2 on v yields a tautologous clause
@@ -683,7 +537,7 @@ bool SatElite::addResolvent(uint32 id, const Clause& lhs, const Clause& rhs) {
 			occurs_[resolvent_[0].var()].unmark();
 			return s->force(resolvent_[0], 0) && s->propagate() && propagateFacts();
 		}
-		clauses_[id]  = Clause::newClause(resolvent_);
+		setClause(id, resolvent_);
 		attach(id, false);
 		return true;
 	}
@@ -695,40 +549,35 @@ unmark:
 }
 
 // extends the model given in assign by the vars that were eliminated
-void SatElite::extendModel(Assignment& assign, LitVec& unconstr) {
+void SatElite::doExtendModel(ValueVec& m, LitVec& unconstr) {
+	const ValueRep value_eliminated = 4u;
 	if (!elimTop_) return;
 	// compute values of eliminated vars / blocked literals by "unit propagating"
 	// eliminated/blocked clauses in reverse order
-	uint32 uv = 0;
-	uint32 us = unconstr.size();
-	uint32 uo = us;
-	if (us) {
-		// flip last unconstraint variable to get "next" model
-		unconstr.back() = ~unconstr.back();
-	}
-	Clause* r    = elimTop_;
+	uint32 uv   = 0;
+	uint32 us   = unconstr.size();
+	Clause* r   = elimTop_;
 	do {
 		Literal x  = (*r)[0];
 		Var last   = x.var();
 		bool check = true;
-		if (!r->blocked()) {
-			// solver set all eliminated vars to true, thus before we can compute the
-			// implied values we first need to set them back to free
-			assign.clearValue(last);
+		if (!r->marked()) {
+			// eliminated var - compute the implied value
+			m[last]  = value_eliminated;
 		}
 		if (uv != us && unconstr[uv].var() == last) {
 			// last is unconstraint w.r.t the current model -
 			// set remembered value
 			check    = false;
-			assign.setValue(last, trueValue(unconstr[uv]));
-			assign.setReason(last, 0);
+			m[last]  = trueValue(unconstr[uv]);
 			++uv;
 		}
 		do {
 			Clause& c = *r;
-			if (assign.value(x.var()) != trueValue(x) && check) {
+			if (m[x.var()] != trueValue(x) && check) {
 				for (uint32 i = 1, end = c.size(); i != end; ++i) {
-					if (assign.value(c[i].var()) != falseValue(c[i])) {
+					ValueRep vi = m[c[i].var()] & 3u;
+					if (vi != falseValue(c[i])) {
 						x = c[i];
 						break;
 					}
@@ -736,35 +585,35 @@ void SatElite::extendModel(Assignment& assign, LitVec& unconstr) {
 				if (x == c[0]) {
 					// all lits != x are false
 					// clause is unit or conflicting
-					assert(c.blocked() || assign.value(x.var()) != falseValue(x));
-					assign.clearValue(x.var());
-					assign.setValue(x.var(), trueValue(x));
-					assign.setReason(x.var(), Antecedent(x));
-					check = false;
+					assert(c.marked() || m[x.var()] != falseValue(x));
+					m[x.var()] = trueValue(x);
+					check      = false;
 				}
 			}
 			r = r->next();
 		} while (r && (x = (*r)[0]).var() == last);
-		if (assign.value(last) == value_free) {
-			// last seems unconstraint w.r.t the model. Assume last to true; remember it
-			// so that we can also enumerate the model containing ~last.
-			assign.setValue(last, value_true);
-			assign.setReason(last, 0);
+		if (m[last] == value_eliminated) {
+			// last seems unconstraint w.r.t the model
+			m[last] |= value_true;
 			unconstr.push_back(posLit(last));	
 		}
 	} while (r);
 	// check whether newly added unconstraint vars are really unconstraint w.r.t the model
 	// or if they are implied by some blocked clause.
-	LitVec::iterator j = unconstr.begin()+uo;
+	LitVec::iterator j = unconstr.begin()+us;
 	for (LitVec::iterator it = j, end = unconstr.end(); it != end; ++it) {
-		if (!it->sign() && assign.reason(it->var()).isNull()) {
+		if ((m[it->var()] & value_eliminated) != 0) {
+			// var is unconstraint - assign to true and remember it
+			// so that we can later enumerate the model containing ~var
+			m[it->var()] = value_true;
 			*j++ = *it;
 		}
 	}
 	unconstr.erase(j, unconstr.end());
-	// pop all vars that were enumerated in both phases
-	while (!unconstr.empty() && unconstr.back().sign()) {
-		unconstr.pop_back();
-	}
 }
-}}
+}
+SatPreprocessor* SatPreParams::create(const SatPreParams& opts) {
+	if (opts.type != 0) { return new SatElite::SatElite(); }
+	return 0;
+}
+}
