@@ -36,10 +36,12 @@ class ClauseCreatorTest : public CppUnit::TestFixture {
 	CPPUNIT_TEST(testCreatorAssertsFirstLit);
 	CPPUNIT_TEST(testCreatorInitsWatches);
 	CPPUNIT_TEST(testCreatorNotifiesHeuristic);
+	CPPUNIT_TEST(testCreatorAddLitBug);
 
 	CPPUNIT_TEST(testCreatorSimplifyRemovesDuplicates);
 	CPPUNIT_TEST(testCreatorSimplifyFindsTauts);
 	CPPUNIT_TEST(testCreatorSimplifyMovesWatch);
+	CPPUNIT_TEST(testCreatorSimplifyBug);
 
 	CPPUNIT_TEST(testCreateNonAssertingLearntClause);
 	CPPUNIT_TEST(testCreateLearntClauseConflict);
@@ -119,7 +121,7 @@ public:
 		solver().assume(~c);
 		solver().assume(~d);
 		solver().propagate();
-		CPPUNIT_ASSERT_EQUAL(true, (bool)creator.startAsserting(Constraint_t::learnt_conflict, a)
+		CPPUNIT_ASSERT_EQUAL(true, (bool)creator.start(Constraint_t::learnt_conflict).add(a)
 			.add(b).add(c).add(d).end());
 
 		CPPUNIT_ASSERT_EQUAL(true, solver().isTrue(a));
@@ -133,7 +135,7 @@ public:
 		solver().assume(~c);
 		solver().assume(~d);
 		
-		creator.startAsserting(Constraint_t::learnt_conflict,a).add(b).add(d).add(c);
+		creator.start(Constraint_t::learnt_conflict).add(a).add(b).add(d).add(c);
 		CPPUNIT_ASSERT_EQUAL(true, (bool)creator.end());	// asserts a
 		solver().undoUntil(2);	// clear a and d
 		solver().assume(~d);		// hopefully d was watched.
@@ -149,18 +151,18 @@ public:
 			std::vector<LitVec::size_type> clSizes_;
 			std::vector<ConstraintType> clTypes_;
 		}*heu = new FakeHeu;
-		solver().setHeuristic(-1, heu);
+		solver().setHeuristic(heu);
 		CPPUNIT_ASSERT_EQUAL(true, (bool)creator.start().add(a).add(b).add(c).add(d).end());
 		ctx.endInit();
 		solver().assume(a);
 		solver().assume(b);
 		solver().propagate();
 
-		CPPUNIT_ASSERT_EQUAL(true, (bool)creator.startAsserting(Constraint_t::learnt_conflict, c)
+		CPPUNIT_ASSERT_EQUAL(true, (bool)creator.start(Constraint_t::learnt_conflict).add(c)
 			.add(~a).add(~b).end());
 
-		CPPUNIT_ASSERT_EQUAL(true, (bool)creator.startAsserting(Constraint_t::learnt_loop, c)
-			.add(~a).add(~b).end());
+		CPPUNIT_ASSERT_EQUAL(true, (bool)creator.start(Constraint_t::learnt_loop).add(c)
+			.add(~a).add(~b).end(0));
 
 		CPPUNIT_ASSERT_EQUAL(uint32(3), (uint32)heu->clSizes_.size());
 		CPPUNIT_ASSERT_EQUAL(uint32(4), (uint32)heu->clSizes_[0]);
@@ -172,8 +174,20 @@ public:
 		CPPUNIT_ASSERT_EQUAL(Constraint_t::learnt_loop, heu->clTypes_[2]);
 	}
 
+	void testCreatorAddLitBug() {
+		creator.start(Constraint_t::learnt_conflict).add(a);
+		solver().assume(b) && solver().propagate();
+		creator.add(~b);
+		solver().assume(c) && solver().propagate();
+		creator.add(~c);
+		solver().assume(d) && solver().propagate();
+		creator.add(~d);
+		creator.end();
+		CPPUNIT_ASSERT_MESSAGE("TODO: Fix me!", creator[0] == a);
+	}
+
 	void testCreatorSimplifyRemovesDuplicates() {
-		creator.start().add(a).add(b).add(c).add(a).add(b).add(d).simplify();
+		creator.start().add(a).add(b).add(c).add(a).add(b).add(d).prepare(true);
 		CPPUNIT_ASSERT(creator.size() == 4);
 		CPPUNIT_ASSERT(creator[0] == a);
 		CPPUNIT_ASSERT(creator[1] == b);
@@ -181,9 +195,7 @@ public:
 		CPPUNIT_ASSERT(creator[3] == d);
 	}
 	void testCreatorSimplifyFindsTauts() {
-		creator.start().add(a).add(b).add(c).add(a).add(b).add(~a).simplify();
-		CPPUNIT_ASSERT(creator.size() <= 4);
-		creator.end();
+		creator.start().add(a).add(b).add(c).add(a).add(b).add(~a).end(ClauseCreator::clause_force_simplify);
 		CPPUNIT_ASSERT(0 == ctx.numConstraints());
 	}
 	void testCreatorSimplifyMovesWatch() {
@@ -191,10 +203,19 @@ public:
 		solver().assume(a); solver().propagate();
 		solver().assume(b); solver().propagate();
 		creator.start(Constraint_t::learnt_loop);
-		creator.add(~a).add(~b).add(~b).add(~a).add(c).add(d).simplify();
+		creator.add(~a).add(~b).add(~b).add(~a).add(c).add(d).prepare(true);
 		CPPUNIT_ASSERT(creator.size() == 4);
 		CPPUNIT_ASSERT_EQUAL(c, creator[0]);
-		CPPUNIT_ASSERT_EQUAL(d, creator[creator.sw()]);
+		CPPUNIT_ASSERT_EQUAL(d, creator[1]);
+	}
+
+	void testCreatorSimplifyBug() {
+		LitVec clause;
+		clause.push_back(negLit(0));
+		clause.push_back(a);
+		clause.push_back(b);
+		ClauseCreator::prepare(*ctx.master(), clause, ClauseCreator::clause_force_simplify);
+		CPPUNIT_ASSERT(clause.size() == 2);
 	}
 
 	void testCreateNonAssertingLearntClause() {
@@ -203,9 +224,9 @@ public:
 		solver().assume(b); solver().propagate();
 		creator.start(Constraint_t::learnt_loop);
 		creator.add(~a).add(~b).add(c).add(d);
-		CPPUNIT_ASSERT_EQUAL(c, creator[0]);
-		CPPUNIT_ASSERT_EQUAL(d, creator[creator.sw()]);
 		CPPUNIT_ASSERT(creator.end());
+		CPPUNIT_ASSERT_EQUAL(c, creator[0]);
+		CPPUNIT_ASSERT_EQUAL(d, creator[1]);
 		CPPUNIT_ASSERT(solver().numLearntConstraints() == 1);
 
 		solver().undoUntil(0);
@@ -213,24 +234,24 @@ public:
 		solver().assume(a); solver().propagate();
 		creator.start(Constraint_t::learnt_loop);
 		creator.add(~a).add(b).add(c);
-		CPPUNIT_ASSERT_EQUAL(b, creator[0]);
-		CPPUNIT_ASSERT_EQUAL(c, creator[creator.sw()]);
 		CPPUNIT_ASSERT(creator.end());
+		CPPUNIT_ASSERT_EQUAL(b, creator[0]);
+		CPPUNIT_ASSERT_EQUAL(c, creator[1]);
 		CPPUNIT_ASSERT(ctx.numLearntShort() == 1);
 	}
 	void testCreateLearntClauseConflict() {
 		ctx.endInit();
 		solver().assume(a); solver().force(b,0); solver().propagate();// level 1
-		solver().assume(c); solver().propagate();										// level 2
-		solver().assume(d); solver().propagate();										// level 3
-		solver().assume(f); solver().propagate();                   // level 4
+		solver().assume(c); solver().propagate();                     // level 2
+		solver().assume(d); solver().propagate();                     // level 3
+		solver().assume(f); solver().propagate();                     // level 4
 
 		creator.start(Constraint_t::learnt_loop);
-		creator.add(~c).add(~a).add(~d).add(~b);								// 2 1 3 1
+		creator.add(~c).add(~a).add(~d).add(~b); // 2 1 3 1
+		CPPUNIT_ASSERT_EQUAL(false, (bool)creator.end());
 		// make sure we watch highest levels, i.e. 3 and 2
 		CPPUNIT_ASSERT_EQUAL(~d, creator[0]);
-		CPPUNIT_ASSERT_EQUAL(~c, creator[creator.sw()]);
-		CPPUNIT_ASSERT_EQUAL(false, (bool)creator.end());
+		CPPUNIT_ASSERT_EQUAL(~c, creator[1]);
 		CPPUNIT_ASSERT(solver().numLearntConstraints() == 0);
 
 		
@@ -240,35 +261,35 @@ public:
 		solver().assume(c); solver().propagate();// level 2
 		creator.start(Constraint_t::learnt_loop);
 		creator.add(~a).add(~c);
-		CPPUNIT_ASSERT_EQUAL(~c, creator[0]);
-		CPPUNIT_ASSERT_EQUAL(~a, creator[creator.sw()]);
 		CPPUNIT_ASSERT_EQUAL(false, (bool)creator.end());
+		CPPUNIT_ASSERT_EQUAL(~c, creator[0]);
+		CPPUNIT_ASSERT_EQUAL(~a, creator[1]);
 		CPPUNIT_ASSERT(ctx.numBinary() == 0);
 	}
 
 	void testCreateNonAssertingLearntClauseAsserting() {
 		ctx.endInit();
 		solver().assume(a); solver().force(b,0); solver().propagate();// level 1
-		solver().assume(c); solver().propagate();										// level 2
+		solver().assume(c); solver().propagate();                     // level 2
 		creator.start(Constraint_t::learnt_loop);
-		creator.add(~c).add(~a).add(d).add(~b);									// 2 1 Free 1
+		creator.add(~c).add(~a).add(d).add(~b);                       // 2 1 Free 1
+		CPPUNIT_ASSERT_EQUAL(true, (bool)creator.end());
 		// make sure we watch the right lits, i.e. d (free) and ~c (highest DL)
 		CPPUNIT_ASSERT_EQUAL(d, creator[0]);
-		CPPUNIT_ASSERT_EQUAL(~c, creator[creator.sw()]);
-		CPPUNIT_ASSERT_EQUAL(true, (bool)creator.end());
+		CPPUNIT_ASSERT_EQUAL(~c, creator[1]);
 		CPPUNIT_ASSERT_EQUAL(true, solver().isTrue(d));
 		CPPUNIT_ASSERT(solver().numLearntConstraints() == 1);
 		// test with a short clause
 		solver().undoUntil(0);
 		solver().reduceLearnts(1.0f);
 		solver().assume(a); solver().force(b,0); solver().propagate();// level 1
-		solver().assume(c); solver().propagate();										// level 2
+		solver().assume(c); solver().propagate();                     // level 2
 		creator.start(Constraint_t::learnt_loop);
-		creator.add(~c).add(~a).add(d);													// 2 1 Free
+		creator.add(~c).add(~a).add(d);                               // 2 1 Free
+		CPPUNIT_ASSERT_EQUAL(true, (bool)creator.end());
 		// make sure we watch the right lits, i.e. d (free) and ~c (highest DL)
 		CPPUNIT_ASSERT_EQUAL(d, creator[0]);
-		CPPUNIT_ASSERT_EQUAL(~c, creator[creator.sw()]);
-		CPPUNIT_ASSERT_EQUAL(true, (bool)creator.end());
+		CPPUNIT_ASSERT_EQUAL(~c, creator[1]);
 		CPPUNIT_ASSERT_EQUAL(true, solver().isTrue(d));
 	}
 
@@ -285,8 +306,6 @@ public:
 		ClauseCreator::Result r = creator.end();
 		CPPUNIT_ASSERT_EQUAL(true, r.ok());
 		CPPUNIT_ASSERT(solver().decisionLevel() == 3);
-
-		creator.start(Constraint_t::learnt_other).add(a).add(~c).end();
 	}
 
 	void testInitWatches() {
@@ -301,7 +320,7 @@ public:
 		solver().assume(d)    && solver().propagate();
 		// aF@2 bF@1 cF@1 dT@3 -> dT@3 aF@2 cF@1 bF@1
 		LitVec temp = cl;
-		ClauseCreator::initWatches(solver(), temp, false);
+		ClauseCreator::prepare(solver(), temp, 0);
 		CPPUNIT_ASSERT(temp[0] == d);
 		CPPUNIT_ASSERT(temp[1] == a);
 		
@@ -337,7 +356,7 @@ public:
 		cl.push_back(~a); cl.push_back(f);
 		cl.push_back(~c); cl.push_back(~b);
 		SharedLiterals* p(SharedLiterals::newShareable(cl, Constraint_t::learnt_other));
-		ClauseCreator::Result r = ClauseCreator::integrate(solver(), p, 0);
+		ClauseCreator::integrate(solver(), p, 0);
 		CPPUNIT_ASSERT(solver().isTrue(f));
 		CPPUNIT_ASSERT(solver().decisionLevel() == solver().rootLevel());
 		solver().popRootLevel();
@@ -355,7 +374,7 @@ public:
 		LitVec cl; 
 		cl.push_back(a);
 		SharedLiterals* p(SharedLiterals::newShareable(cl, Constraint_t::learnt_other));
-		ClauseCreator::Result r = ClauseCreator::integrate(solver(), p, 0);
+		ClauseCreator::integrate(solver(), p, 0);
 		CPPUNIT_ASSERT(solver().isTrue(a));
 		CPPUNIT_ASSERT(solver().decisionLevel() == 0);
 	}
@@ -386,7 +405,7 @@ public:
 		LitVec cl;
 		cl.push_back(~a); cl.push_back(~c); cl.push_back(~b);
 		SharedLiterals* p(SharedLiterals::newShareable(cl, Constraint_t::learnt_other));
-		ClauseCreator::Result r = ClauseCreator::integrate(solver(), p, 0);
+		ClauseCreator::integrate(solver(), p, 0);
 		CPPUNIT_ASSERT(solver().decisionLevel() == uint32(2));
 	}
 
@@ -400,7 +419,7 @@ public:
 		LitVec cl;
 		cl.push_back(~a); cl.push_back(~c); cl.push_back(~b);
 		SharedLiterals* p(SharedLiterals::newShareable(cl, Constraint_t::learnt_other));
-		ClauseCreator::Result r = ClauseCreator::integrate(solver(), p, 0);
+		ClauseCreator::integrate(solver(), p, 0);
 		CPPUNIT_ASSERT(solver().decisionLevel() == uint32(3));
 		solver().popRootLevel();
 		solver().backtrack() && solver().propagate();
@@ -474,7 +493,7 @@ public:
 		cl.push_back(a);
 		SharedLiterals* p(SharedLiterals::newShareable(cl,Constraint_t::learnt_other));
 		CPPUNIT_ASSERT((ClauseCreator::status(solver(), p->begin(), p->end()) & ClauseCreator::status_unit) != 0);
-		ClauseCreator::integrate(solver(), p, ClauseCreator::clause_known_order);
+		ClauseCreator::integrate(solver(), p, ClauseCreator::clause_no_prepare);
 		CPPUNIT_ASSERT(solver().isTrue(a));
 	}
 
@@ -484,7 +503,7 @@ public:
 		SharedLiterals* p(SharedLiterals::newShareable(cl,Constraint_t::learnt_other));
 		solver().assume(~a) && solver().propagate();
 		solver().force(~b,0)&& solver().propagate();
-		CPPUNIT_ASSERT(ClauseCreator::status(solver(), p->begin(), p->end()) == ClauseCreator::status_conflicting);
+		CPPUNIT_ASSERT((ClauseCreator::status(solver(), p->begin(), p->end()) & ClauseCreator::status_unsat) != 0);
 		ClauseCreator::Result r = ClauseCreator::integrate(solver(), p, ClauseCreator::clause_not_conflict);
 		CPPUNIT_ASSERT(r.ok() == false);
 		CPPUNIT_ASSERT(r.local == 0);
@@ -567,6 +586,7 @@ public:
 
 
 	void testFactsAreRemovedFromLearnt() {
+		ctx.enableStats(1);
 		ctx.addUnary(a);
 		ctx.endInit();
 		solver().assume(~b) && solver().propagate();
@@ -574,7 +594,7 @@ public:
 		creator.add(b).add(c).add(~a).end();
 
 		CPPUNIT_ASSERT(1u == ctx.numLearntShort());
-		CPPUNIT_ASSERT(ctx.master()->stats.lits[0] == 2);
+		CPPUNIT_ASSERT(ctx.master()->stats.extra->lits[0] == 2);
 	}
 private:
 	const Solver& solver() const { return *ctx.master(); }
