@@ -110,15 +110,15 @@ public:
 		ClauseHead* cl1;
 		solver->add(cl1 = createClause(2,2));
 		CPPUNIT_ASSERT_EQUAL(2, countWatches(*solver, cl1, clLits) );
-		ClauseHead* cl2 = Clause::newLearntClause(*solver, clLits, ClauseInfo(Constraint_t::learnt_conflict));
+		ClauseHead* cl2 = createClause(clLits, Constraint_t::learnt_conflict);
 		solver->add(cl2);
 		CPPUNIT_ASSERT_EQUAL(2,  countWatches(*solver, cl2, clLits));
 	}
 
 	void testClauseTypes() {
 		Clause* cl1 = createClause(2, 2);
-		LearntConstraint* cl2 = Clause::newLearntClause(*solver, clLits, ClauseInfo(Constraint_t::learnt_conflict));
-		LearntConstraint* cl3 = Clause::newLearntClause(*solver, clLits, ClauseInfo(Constraint_t::learnt_loop));
+		LearntConstraint* cl2 = createClause(clLits, ClauseInfo(Constraint_t::learnt_conflict));
+		LearntConstraint* cl3 = createClause(clLits, ClauseInfo(Constraint_t::learnt_loop));
 		CPPUNIT_ASSERT_EQUAL(Constraint_t::static_constraint, cl1->type());
 		CPPUNIT_ASSERT_EQUAL(Constraint_t::learnt_conflict, cl2->type());
 		CPPUNIT_ASSERT_EQUAL(Constraint_t::learnt_loop, cl3->type());
@@ -134,8 +134,8 @@ public:
 		lit.push_back(posLit(3));
 		lit.push_back(posLit(4));
 		uint32 exp = 258;
-		ClauseHead* cl1 = Clause::newLearntClause(*solver, lit, ClauseInfo(Constraint_t::learnt_conflict).setActivity(exp));
-		ClauseHead* cl2 = Clause::newLearntClause(*solver, lit, ClauseInfo(Constraint_t::learnt_loop).setActivity(exp));
+		ClauseHead* cl1 = createClause(lit, ClauseInfo(Constraint_t::learnt_conflict).setActivity(exp));
+		ClauseHead* cl2 = createClause(lit, ClauseInfo(Constraint_t::learnt_loop).setActivity(exp));
 		solver->add(cl1);
 		solver->add(cl2);
 		while ( exp != 0 ) {
@@ -206,7 +206,6 @@ public:
 		for (int i = 0; i != 100; ++i) {
 			SharedContext cc;
 			solver = cc.master();
-			solver->strategies().initWatches = SolverStrategies::watch_rand;
 			for (int j = 0; j < 12; ++j) { cc.addVar(Var_t::atom_var); }
 			cc.startAddConstraints(1);
 			Clause* c;
@@ -221,7 +220,7 @@ public:
 		clLits.push_back(posLit(3));
 		clLits.push_back(posLit(4));
 		ctx.endInit();
-		ClauseHead* cl1 = Clause::newLearntClause(*solver, clLits, ClauseInfo(Constraint_t::learnt_conflict));
+		ClauseHead* cl1 = createClause(clLits, ClauseInfo(Constraint_t::learnt_conflict));
 		solver->addLearnt(cl1, (uint32)clLits.size());
 		solver->assume(~clLits[0]);
 		solver->propagate();
@@ -336,7 +335,7 @@ public:
 		clLits.push_back(b1);
 		clLits.push_back(b2);
 		clLits.push_back(b3);
-		c = Clause::newLearntClause(*solver, clLits, ClauseInfo());
+		c = createClause(clLits, ClauseInfo());
 		CPPUNIT_ASSERT_EQUAL(false, c->strengthen(*solver, b2).second);
 		CPPUNIT_ASSERT(c->size() == 5);
 		CPPUNIT_ASSERT_EQUAL(false, c->strengthen(*solver, a3).second);
@@ -351,11 +350,9 @@ public:
 		Literal b = posLit(ctx.addVar( Var_t::atom_var ));
 		Literal x = posLit(ctx.addVar( Var_t::atom_var ));
 		Literal y = posLit(ctx.addVar( Var_t::atom_var ));
-		ctx.requestTagLiteral(); 
 		ctx.startAddConstraints();
-		Literal a = ctx.tagLiteral();
-		solver->assume(a) && solver->propagate();
-		solver->pushRootLevel(1);
+		ctx.endInit();
+		Literal a = posLit(solver->pushTagVar(true));
 		solver->assume(x) && solver->propagate();
 		solver->assume(y) && solver->propagate();
 		solver->setBacktrackLevel(solver->decisionLevel());
@@ -363,7 +360,7 @@ public:
 		clLits.push_back(b);
 		clLits.push_back(~a);
 		ClauseInfo extra(Constraint_t::learnt_conflict); extra.setTagged(true);
-		ClauseHead* c = ClauseCreator::create(*solver, clLits, extra).local;
+		ClauseHead* c = ClauseCreator::create(*solver, clLits, 0, extra).local;
 		CPPUNIT_ASSERT(c->size() == 2);
 		CPPUNIT_ASSERT(solver->isTrue(b) && solver->reason(b).constraint() == c);
 		solver->backtrack() && solver->propagate();
@@ -379,15 +376,15 @@ public:
 	}
 
 	void testStrengthenContracted() {
-		solver->strategies().compress = 4;
 		ctx.endInit();
-		ClauseCreator creator(solver);
-		creator.startAsserting(Constraint_t::learnt_conflict, a1);
+		LitVec lits;
+		lits.push_back(a1);
 		for (uint32 i = 2; i <= 12; ++i) {
 			solver->assume(negLit(i));
-			creator.add(posLit(i));
+			lits.push_back(posLit(i));
 		}
-		ClauseHead* c = creator.end().local;
+		solver->strategies().compress = 4;
+		ClauseHead* c = ClauseCreator::create(*solver, lits, 0, Constraint_t::learnt_conflict).local;
 		uint32 si = c->size();
 		c->strengthen(*solver, posLit(12));
 		solver->undoUntil(solver->decisionLevel()-1);
@@ -414,7 +411,6 @@ public:
 	}
 
 	void testStrengthenBug() {
-		solver->strategies().compress = 6;
 		ctx.endInit();
 		LitVec clause;
 		clause.push_back(a1);
@@ -422,7 +418,7 @@ public:
 			solver->assume(negLit(8-i)) && solver->propagate();
 			clause.push_back(posLit(i));
 		}
-		ClauseHead* c = Clause::newContractedClause(*solver, clause, ClauseInfo(Constraint_t::learnt_conflict), 5, true);
+		ClauseHead* c = Clause::newContractedClause(*solver, ClauseRep::create(&clause[0], (uint32)clause.size(), ClauseInfo(Constraint_t::learnt_conflict)) , 5, true);
 		solver->addLearnt(c, 5);
 		uint32 si = c->size();
 		CPPUNIT_ASSERT(si == 5);
@@ -436,7 +432,6 @@ public:
 	}
 
 	void testStrengthenContractedNoExtend() {
-		solver->strategies().compress = 4;
 		ctx.endInit();
 		LitVec clause;
 		clause.push_back(a1);
@@ -444,7 +439,8 @@ public:
 			solver->assume(negLit(8-i)) && solver->propagate();
 			clause.push_back(posLit(i));
 		}
-		ClauseHead* c = Clause::newContractedClause(*solver, clause, ClauseInfo(Constraint_t::learnt_conflict), 4, false);
+		ClauseRep   x = ClauseRep::create(&clause[0], (uint32)clause.size(), ClauseInfo(Constraint_t::learnt_conflict));
+		ClauseHead* c = Clause::newContractedClause(*solver, x, 4, false);
 		solver->addLearnt(c, 4);
 		CPPUNIT_ASSERT(c->size() == 4);
 		c->strengthen(*solver, posLit(2));
@@ -457,10 +453,9 @@ public:
 		Var a = ctx.addVar( Var_t::atom_var );
 		Var b = ctx.addVar( Var_t::atom_var );
 		Var c = ctx.addVar( Var_t::atom_var );
-		ctx.requestTagLiteral();
 		ctx.startAddConstraints();
-		Literal tag = ctx.tagLiteral();
-		solver->assume(tag) && solver->propagate();
+		ctx.endInit();
+		Literal tag = posLit(solver->pushTagVar(true));
 		solver->assume(posLit(a)) && solver->propagate();
 		solver->assume(posLit(b)) && solver->propagate();
 		ClauseCreator cc(solver);
@@ -481,11 +476,9 @@ public:
 		Literal c = posLit(ctx.addVar( Var_t::atom_var ));
 		Literal d = posLit(ctx.addVar( Var_t::atom_var ));
 		Literal x = posLit(ctx.addVar( Var_t::atom_var ));
-		ctx.requestTagLiteral(); 
 		ctx.startAddConstraints();
-		Literal a = ctx.tagLiteral();
-		solver->assume(a) && solver->propagate();
-		solver->pushRootLevel(1);
+		ctx.endInit();
+		Literal a = posLit(solver->pushTagVar(true));
 		solver->assume(b) && solver->propagate();
 		solver->force(c, 0) && solver->propagate();
 		solver->assume(x) && solver->propagate();
@@ -505,19 +498,15 @@ public:
 		Var a = ctx.addVar( Var_t::atom_var );
 		Var b = ctx.addVar( Var_t::atom_var );
 		Var c = ctx.addVar( Var_t::atom_var );
-		ctx.requestTagLiteral(); // d
 		ctx.startAddConstraints();
-		Literal tag = ctx.tagLiteral();
-		solver->assume(tag) && solver->propagate();
-		solver->pushRootLevel(1);
+		ctx.endInit();
+		Literal tag = posLit(solver->pushTagVar(true));
 		ClauseCreator cc(solver);
 		// ~a ~b ~c ~tag
 		cc.start(Constraint_t::learnt_conflict).add(negLit(a)).add(negLit(b)).add(negLit(c)).add(~tag);
 		ClauseHead* clause = cc.end().local;
 		
-		// learn c
-		solver->addUnary(posLit(c), Constraint_t::static_constraint);
-		
+		solver->force(posLit(c));
 		CPPUNIT_ASSERT_EQUAL(false, clause->strengthen(*solver, negLit(c)).second);
 	}
 	
@@ -561,23 +550,22 @@ public:
 	}
 	
 	void testContraction() {
-		solver->strategies().compress = 6;
 		ctx.endInit();
-		ClauseCreator creator(solver);
-		creator.startAsserting(Constraint_t::learnt_conflict, a1);
+		LitVec lits(1, a1);
 		for (uint32 i = 2; i <= 12; ++i) {
 			solver->assume(negLit(i));
-			creator.add(posLit(i));
+			lits.push_back(posLit(i));
 		}
-		ClauseHead* cl = creator.end().local;
+		solver->strategies().compress = 6;
+		ClauseHead* cl = ClauseCreator::create(*solver, lits, 0, Constraint_t::learnt_conflict).local;
 		uint32  s1 = cl->size();
-		CPPUNIT_ASSERT(s1 < creator.lits().size());
+		CPPUNIT_ASSERT(s1 < lits.size());
 		LitVec r;
 		solver->reason(a1, r);
-		CPPUNIT_ASSERT( r.size() == creator.lits().size()-1 );
+		CPPUNIT_ASSERT( r.size() == lits.size()-1 );
 
 		solver->undoUntil(0);
-		CPPUNIT_ASSERT(cl->size() == creator.lits().size());
+		CPPUNIT_ASSERT(cl->size() == lits.size());
 	}
 
 	void testNewContractedClause() {
@@ -591,7 +579,8 @@ public:
 			// (false) tail
 			clLits.push_back(posLit(i));
 		}
-		ClauseHead* c = Clause::newContractedClause(*solver, clLits, ClauseInfo(Constraint_t::learnt_conflict), 3, false);
+		ClauseRep   x = ClauseRep::create(&clLits[0], (uint32)clLits.size(), ClauseInfo(Constraint_t::learnt_conflict));
+		ClauseHead* c = Clause::newContractedClause(*solver, x, 3, false);
 		solver->addLearnt(c, static_cast<uint32>(clLits.size()));
 		CPPUNIT_ASSERT(c->size() < clLits.size());
 
@@ -630,10 +619,8 @@ public:
 	}
 
 	void testClone() {
-		Solver solver2;
-		ctx.setSolvers(2);
-		ctx.endInit();
-		ctx.attach(solver2);
+		Solver& solver2 = ctx.addSolver();
+		ctx.endInit(true);
 		ClauseHead* c      = createClause(3, 3);
 		ClauseHead* clone  = (ClauseHead*)c->cloneAttach(solver2);
 		LitVec lits;
@@ -682,10 +669,15 @@ public:
 		solver->force( ~a2, 0 );
 		solver->propagate();
 		CPPUNIT_ASSERT_EQUAL(false, lf->simplify(*solver));
-		solver->force( ~a3, 0 );
+		solver->assume(a3);
+		solver->propagate();
+		solver->backtrack();
 		solver->propagate();
 		CPPUNIT_ASSERT_EQUAL(true, lf->simplify(*solver));
 		CPPUNIT_ASSERT_EQUAL(false, solver->hasWatch(~b3, lf));
+		CPPUNIT_ASSERT_EQUAL(false, solver->hasWatch(a1, lf));
+		CPPUNIT_ASSERT_EQUAL(false, solver->hasWatch(a2, lf));
+		CPPUNIT_ASSERT_EQUAL(false, solver->hasWatch(a3, lf));
 		solver->reduceLearnts(1.0f);
 	}
 
@@ -696,7 +688,7 @@ public:
 		solver->force( ~b1, 0 );
 		solver->propagate();
 		CPPUNIT_ASSERT_EQUAL(true, lf->simplify(*solver));
-		CPPUNIT_ASSERT(3u == solver->sharedContext()->numTernary());
+		CPPUNIT_ASSERT(3u == solver->sharedContext()->numLearntShort());
 	}
 
 	void testSimplifyLFRemovesFalseAtoms() {
@@ -722,17 +714,9 @@ public:
 		solver->undoUntil(0);
 		solver->force( a1,0 );
 		solver->propagate();
-		CPPUNIT_ASSERT_EQUAL(false, lf->simplify(*solver));
-		CPPUNIT_ASSERT(5 == lf->size());
-
-		solver->force( a3,0 );
-		solver->propagate();
-		CPPUNIT_ASSERT_EQUAL(false, lf->simplify(*solver));
-		CPPUNIT_ASSERT(4 == lf->size());
-
-		solver->force( a2,0 );
-		solver->propagate();
 		CPPUNIT_ASSERT_EQUAL(true, lf->simplify(*solver));
+		
+		CPPUNIT_ASSERT(1u == solver->sharedContext()->numLearntShort());
 	}
 
 	void testLoopFormulaPropagateBody() {
@@ -791,6 +775,10 @@ public:
 		solver->propagate();
 		solver->assume( ~b1 );
 		solver->propagate();
+		
+		solver->assume( ~a1 );
+		solver->propagate();
+		
 		solver->assume( ~b2 );
 		solver->propagate();
 
@@ -798,9 +786,9 @@ public:
 		CPPUNIT_ASSERT_EQUAL( true, solver->isTrue(~a2) );
 		CPPUNIT_ASSERT_EQUAL( true, solver->isTrue(~a3) );
 		
-		CPPUNIT_ASSERT_EQUAL( Antecedent::generic_constraint, solver->reason(~a1).type() );
+		CPPUNIT_ASSERT_EQUAL( Antecedent::generic_constraint, solver->reason(~a2).type() );
 		LitVec r;
-		solver->reason(~a1, r);
+		solver->reason(~a2, r);
 		CPPUNIT_ASSERT_EQUAL( LitVec::size_type(3), r.size() );
 		CPPUNIT_ASSERT(std::find(r.begin(), r.end(), ~b1) != r.end());
 		CPPUNIT_ASSERT(std::find(r.begin(), r.end(), ~b2) != r.end());
@@ -937,11 +925,11 @@ public:
 		solver->assume(~body2);
 		solver->assume(~body3);
 		solver->propagate();
-		Literal bodies[3] = {body1, body2, body3 };
-		LoopFormula* lf = LoopFormula::newLoopFormula(*solver, &bodies[0], 3, 2, 1, Activity(0,10));
+		Literal lits[4] = { ~a1, body3, body2, body1 };
+		
+		LoopFormula* lf = LoopFormula::newLoopFormula(*solver, ClauseRep::prepared(lits, 4), lits, 1);
 		solver->addLearnt(lf, lf->size());
 		solver->force(~a1, lf);
-		lf->addAtom(~a1, *solver);
 		solver->propagate();
 		solver->undoUntil(solver->decisionLevel()-2);
 		CPPUNIT_ASSERT_EQUAL(true, solver->assume(~body3) && solver->propagate());
@@ -1007,6 +995,10 @@ private:
 		res+="]";
 		return res;
 	}
+	Clause* createClause(LitVec& lits, const ClauseInfo& info) {
+		uint32 flags = ClauseCreator::clause_explicit | ClauseCreator::clause_no_add | ClauseCreator::clause_no_prepare;
+		return (Clause*)ClauseCreator::create(*solver, lits, flags, info).local;
+	}
 	Clause* createClause(int pos, int neg, ConstraintType t = Constraint_t::static_constraint) {
 		clLits.clear();
 		int size = pos + neg;
@@ -1019,7 +1011,7 @@ private:
 			lit[i] = negLit(i+1);
 			clLits.push_back(lit[i]);
 		}
-		return (Clause*)Clause::newClause(*solver, &lit[0], (uint32)lit.size(), ClauseInfo(t));
+		return createClause(clLits, ClauseInfo(t));
 	}
 
 	Clause* createRandomClause(int size) {
@@ -1033,15 +1025,12 @@ private:
 		solver->assume(~b2);
 		solver->assume(~b3);
 		solver->propagate();
-		Literal bodies[3] = {b1, b2, b3 };
-		LoopFormula* lf = LoopFormula::newLoopFormula(*solver, &bodies[0], 3, 2, 3, Activity(0,10));
+		Literal lits[] = { ~a1, b3, b2, b1, ~a1, ~a2, ~a3 };
+		LoopFormula* lf = LoopFormula::newLoopFormula(*solver, ClauseRep::prepared(lits, 4), lits+4, 3);
 		solver->addLearnt(lf, lf->size());
 		solver->force(~a1, lf);
 		solver->force(~a2, lf);
 		solver->force(~a3, lf);
-		lf->addAtom(~a1, *solver);
-		lf->addAtom(~a2, *solver);
-		lf->addAtom(~a3, *solver);
 		solver->propagate();
 		return lf;
 	}
