@@ -62,6 +62,7 @@ class CliTest : public CppUnit::TestFixture {
 	CPPUNIT_TEST(testSetIntegrate);
 #endif
 	CPPUNIT_TEST(testConfigQueryStrValues);
+	CPPUNIT_TEST(testSetOptBound);
 	CPPUNIT_TEST_SUITE_END();
 public:
 	void testConfigArgv() {
@@ -113,8 +114,16 @@ public:
 	void testConfigImplicitCreateSolver() {
 		ClaspCliConfig config;
 		CPPUNIT_ASSERT(config.numSolver() == 1);
-		CPPUNIT_ASSERT(config.setValue("solver.1.heuristic", "berkmin") && config.solver(1).heuId == Heuristic_t::heu_berkmin);
+		// solver option
+		CPPUNIT_ASSERT(config.setValue("solver.1.heuristic", "berkmin"));
 		CPPUNIT_ASSERT(config.numSolver() == 2);
+		CPPUNIT_ASSERT(config.solver(1).heuId == Heuristic_t::heu_berkmin);
+		
+		CPPUNIT_ASSERT(config.setValue("solver.17.heuristic", "unit"));
+		CPPUNIT_ASSERT(config.numSolver() == 18);
+		for (uint32 i = 0; i != config.numSolver(); ++i) {
+			CPPUNIT_ASSERT_EQUAL_MESSAGE("solver id not set", i, config.solver(i).id);
+		}
 	}
 	void testConfigImplicitCreateTester() {
 		ClaspCliConfig config;
@@ -525,6 +534,24 @@ public:
 		CPPUNIT_ASSERT_THROW(config.getValue("enum"), std::logic_error);
 		CPPUNIT_ASSERT_THROW(config.getValue("tester.solve.opt_bound"), std::logic_error);
 	}
+
+	void testSetOptBound() {
+		ClaspCliConfig config;
+		std::string bound = config.getValue("solve.opt_bound");
+		CPPUNIT_ASSERT(bound == "no");
+		CPPUNIT_ASSERT_EQUAL(true, config.setValue("solve.opt_bound", "100"));
+		CPPUNIT_ASSERT((bound = config.getValue("solve.opt_bound")) == "100");
+		CPPUNIT_ASSERT_EQUAL(true, config.setValue("solve.opt_bound", "50,20"));
+		bound = config.getValue("solve.opt_bound");
+		CPPUNIT_ASSERT_EQUAL(bound, std::string("50,20"));
+		CPPUNIT_ASSERT_EQUAL(true, config.setValue("solve.opt_bound", "no"));
+		CPPUNIT_ASSERT((bound = config.getValue("solve.opt_bound")) == "no");
+
+		CPPUNIT_ASSERT_EQUAL(true, config.setValue("solve.opt_bound", "50,20"));
+		CPPUNIT_ASSERT_EQUAL(false, config.setValue("solve.opt_bound", "a,b"));
+		bound = config.getValue("solve.opt_bound");
+		CPPUNIT_ASSERT_EQUAL(bound, std::string("50,20"));
+	}
 private:
 	bool isValidOption(const ClaspCliConfig& c, const std::string& k) const {
 		return ClaspCliConfig::isLeafKey(c.getKey(ClaspCliConfig::KEY_ROOT, k.c_str()));
