@@ -58,6 +58,9 @@ class DecisionHeuristicTest : public CppUnit::TestFixture {
 	CPPUNIT_TEST(testDomIncPrio);
 	CPPUNIT_TEST(testDomReinit);
 	CPPUNIT_TEST(testDomMinBug);
+	CPPUNIT_TEST(testDomSignPrio);
+	CPPUNIT_TEST(testDomPrefixBug);
+	CPPUNIT_TEST(testDomSameVar);
 	CPPUNIT_TEST_SUITE_END();
 public:
 	void testTrivial() {
@@ -654,6 +657,70 @@ public:
 
 		CPPUNIT_ASSERT(ctx.symbolTable().domLits.size() == 2);
 	}
+	void testDomSignPrio() {
+		SharedContext ctx;
+		DecisionHeuristic* dom;
+		ctx.master()->setHeuristic(dom = new DomainHeuristic);
+		Solver& s = *ctx.master();
+
+		LogicProgram api;
+		api.start(ctx).setAtomName(1, "a").setAtomName(2, "b")
+			.startRule(Asp::CHOICERULE).addHead(1).addHead(2).endRule()
+			;
+		// implicit prio
+		addDomRule(api, "_heuristic(a,sign,100)");
+		addDomRule(api, "_heuristic(a,sign,-10)");
+		// explicit prio
+		addDomRule(api, "_heuristic(b,sign,100,100)");
+		addDomRule(api, "_heuristic(b,sign,-10,10)");
+
+		CPPUNIT_ASSERT_EQUAL(true, api.endProgram() && ctx.endInit());
+
+		Literal a = api.getLiteral(1);
+		Literal b = api.getLiteral(2);
+
+		Literal x = dom->doSelect(s);
+		s.assume(x) && s.propagate();
+		Literal y = dom->doSelect(s);
+		s.assume(y) && s.propagate();
+		CPPUNIT_ASSERT(x != y);
+		CPPUNIT_ASSERT(x == a || x == b);
+		CPPUNIT_ASSERT(y == a || y == b);
+	}
+	void testDomPrefixBug() {
+		DomainHeuristic* dom = new DomainHeuristic;
+		SharedContext ctx;
+		ctx.master()->setHeuristic(dom);
+		Solver& s = *ctx.master();
+		LogicProgram api;
+		api.start(ctx).setAtomName(1, "a").setAtomName(2,"ab").setAtomName(3,"abc").startRule(Asp::CHOICERULE).addHead(1).addHead(2).addHead(3).endRule();
+		addDomRule(api, "_heuristic(ab,level,1)");
+		addDomRule(api, "_heuristic(ab,factor,2)");
+		CPPUNIT_ASSERT_EQUAL(true, api.endProgram() && ctx.endInit());
+		const DomScore& scA = dom->score(api.getLiteral(1).var());
+		const DomScore& scB = dom->score(api.getLiteral(2).var());
+		const DomScore& scC = dom->score(api.getLiteral(3).var());
+		CPPUNIT_ASSERT(scA.level < scB.level);
+		CPPUNIT_ASSERT(scC.level < scB.level);
+		CPPUNIT_ASSERT(scA.factor < scB.factor);
+		CPPUNIT_ASSERT(scC.factor < scB.factor);
+	}
+	void testDomSameVar() {
+		DecisionHeuristic* dom = new DomainHeuristic;
+		SharedContext ctx;
+		ctx.master()->setHeuristic(dom);
+		Solver& s = *ctx.master();
+		LogicProgram api;
+		api.start(ctx).setAtomName(1, "a").setAtomName(2, "b")
+			.startRule().addHead(1).addToBody(2, false).endRule()
+			.startRule().addHead(2).addToBody(1, false).endRule();
+		addDomRule(api, "_heuristic(a,true,2)");
+		addDomRule(api, "_heuristic(b,true,1)");
+		CPPUNIT_ASSERT_EQUAL(true, api.endProgram() && ctx.endInit());
+		CPPUNIT_ASSERT(api.getLiteral(1).var() == api.getLiteral(2).var());
+		Literal x = dom->doSelect(s);
+		CPPUNIT_ASSERT(x == api.getLiteral(1));
+	}
 	void addDomRule(LogicProgram& prg, const char* heu, Literal pre = posLit(0)) {
 		Var h = prg.newAtom();
 		prg.setAtomName(h, heu);
