@@ -12,8 +12,7 @@
 //  GNU General Public License for more details.
 //
 //  You should have received a copy of the GNU General Public License
-//  along with this file; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+//  along with this program. If not, see <http://www.gnu.org/licenses/>.
 //
 //
 // NOTE: ProgramOptions is inspired by Boost.Program_options
@@ -54,7 +53,8 @@ public:
 	enum Type {
 		duplicate_option,
 		unknown_option,
-		ambiguous_option
+		ambiguous_option,
+		unknown_group,
 	};
 	ContextError(const std::string& ctx, Type t, const std::string& key, const std::string& desc = "");
 	~ContextError() throw () {}
