@@ -12,8 +12,7 @@
 //  GNU General Public License for more details.
 //
 //  You should have received a copy of the GNU General Public License
-//  along with this file; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+//  along with this program. If not, see <http://www.gnu.org/licenses/>.
 //
 //
 // NOTE: ProgramOptions is inspired by Boost.Program_options
@@ -122,6 +121,7 @@ public:
 
 	//! Adds option to this group.
 	void addOption(std::auto_ptr<Option> option);
+	void addOption(const SharedOptPtr& option);
 
 	void setDescriptionLevel(DescriptionLevel level) { level_ = level; }
 
